@@ -15,30 +15,33 @@ class AMQ_BrokerConfig:
     uid = 'von'
     password = 'von1970'
 
+
+class RabbitMq_Message():
+
+    def __init__(self, ch, method, body) -> None:
+        self.ch = ch
+        self.method = method
+        self.body = body
+
 class RabbitMq_Subscriber():
     def __init__(self, queue_name: str) -> None:
         self.queue_name = queue_name
-        self.channel = None
-        self.prefeched_message = None
-        self.delivery_tag = 0
+        self.unacked_messages = []
 
     def CopyToFetchedMessage(self, ch, method, properties, body):
-        # print(ch, method ,'\n',  method.routing_key,'\n',  properties, '\n', body)
-        # print('\n\n\n', properties)
-
-        # self.channel.basic_ack(delivery_tag=method.delivery_tag)
-        self.channel = ch
-        if self.prefeched_message is None:
-            self.delivery_tag = method.delivery_tag
-            self.channel.basic_ack(delivery_tag=self.delivery_tag)
-            self.prefeched_message = body
+        unacked = RabbitMq_Message(ch,method,body)
+        self.unacked_messages.append(unacked)
 
     def FetchMessage(self) -> str:
-        result = self.prefeched_message
-        if self.prefeched_message is not None:
-            # self.channel.basic_ack(delivery_tag=self.delivery_tag)
-            # print('act to brocker')
-            self.prefeched_message = None
+        if len(self.unacked_messages) > 0:
+            result = self.unacked_messages[0].body
+            ch = self.unacked_messages[0].ch
+            tag = self.unacked_messages[0].method.delivery_tag
+            ch.basic_ack(tag)
+            self.unacked_messages.pop(0)
+            # print('poped')
+        else:
+            result = None
         return result
 
 class RabbitMqAgent():
@@ -46,27 +49,18 @@ class RabbitMqAgent():
     To learn:  What is channel indeed ?
     '''
     def __init__(self) -> None:
-        # self.declaed_queues=[]
         self.subscribers=[RabbitMq_Subscriber('Nothing')]
         self.subscribers.clear
-        self.prefeched_message = None
 
     def connect_to_broker(self, broker_config: AMQ_BrokerConfig) -> None:
         self.serverConfig = broker_config
         self.reconnect_to_broker()
         self.prefeched_message = None
         self.delivery_tag = None
-        # self.blocking_connection = None
 
-    def SpinOnce(self, time_limit=0.1):
+    def process_data_events(self, time_limit=0):
         self.blocking_connection.process_data_events(time_limit)
     
-    # def FetchMessage(self) -> str:
-    #     result = self.prefeched_message
-    #     if self.prefeched_message is not None:
-    #         self.channel.basic_ack(delivery_tag=self.delivery_tag)
-    #         self.prefeched_message = None
-    #     return result
 
     def reconnect_to_broker(self):
         broker_config = self.serverConfig
@@ -79,7 +73,7 @@ class RabbitMqAgent():
             print("Start to connect to RabbitMQ broker.")
             self.blocking_connection = pika.BlockingConnection(parameters)
             self.channel = self.blocking_connection.channel()
-            self.channel.basic_qos(prefetch_count=1)
+            self.channel.basic_qos(prefetch_count=10)
             print("Connected RabbitMQ broker!")
         except Exception as e:
             print(e)
@@ -128,7 +122,6 @@ class RabbitMqAgent():
         return None
 
     def fetch_message(self, queue_name:str) -> str:
-        # self.SpinOnce()
         subscriber = self.__FindSubscriber(queue_name)
         if subscriber is None:
             print("no subscriber is found")
@@ -136,29 +129,17 @@ class RabbitMqAgent():
         else:
             return subscriber.FetchMessage()
 
-    # def CopyToFetchedMessage(self, ch, method, properties, body):
-    #     # print('RabbitMqAgent::callback_example()  mq Received ' ,  method.routing_key, body)
-    #     # self.channel.basic_ack(delivery_tag=method.delivery_tag)
-    #     self.prefeched_message = body
-    #     self.delivery_tag = method.delivery_tag
-
-
     def Subscribe(self, queue_name:str, callback=None):
         '''
         call back examole def callback_main(self, ch, method, properties, body):
         If using FetchMessage(), ignore the callback
         '''
-        # if not (queue_name in self.declaed_queues):
-            # self.channel.queue_declare(queue=queue_name, durable=True)
-            # self.declaed_queues.append(queue_name)
         new_subscriber = RabbitMq_Subscriber(queue_name)
         new_subscriber.channel = self.channel
         self.subscribers.append(new_subscriber)
         var_callback = callback
         if callback is None:
-            # var_callback = self.CopyToFetchedMessage
             var_callback = new_subscriber.CopyToFetchedMessage
-        # self.channel.basic_consume(queue=queue_name, on_message_callback=var_callback, auto_ack=False)
         new_subscriber.channel.basic_consume(queue=queue_name, on_message_callback=var_callback, auto_ack=False)
 
     def RabbitMQ_publish_tester(self):
@@ -184,27 +165,31 @@ if __name__ == '__main__':
     # img = cv2.imread("nocommand.jpg")
     # g_amq.publish_cv_image("test" , img)
 
-    g_amq.Subscribe(queue_name='twh_221109_deposit')
-    g_amq.Subscribe(queue_name='twh_221109_withdraw')
+    g_amq.Subscribe(queue_name='twh_deposit')
+    g_amq.Subscribe(queue_name='twh_withdraw')
     count = 0
+    g_amq.process_data_events()
     while True:
-        g_amq.SpinOnce()
+        g_amq.process_data_events()
+        g_amq.process_data_events()
         # time.sleep(0.9)
-        xx = g_amq.fetch_message('twh_221109_deposit')
+        xx = g_amq.fetch_message('twh_deposit')
         # xx = ss.FetchMessage()
         if xx is not None:
             print(xx)
-            # time.sleep(1)
+            time.sleep(1)
 
-        xx = g_amq.fetch_message('twh_221109_withdraw')
+        xx = g_amq.fetch_message('twh_withdraw')
         if xx is not None:
+            # g_amq.process_data_events()
             if count>1:
                 print("withdraw is empty................................", count)
             count =0
-            print(xx)
-            # time.sleep(1)
+            # print('final fetchd.', xx)
         else:
+            # print("fetched   None", count)
             count += 1
+        # time.sleep(1)
 
 
     
\ No newline at end of file
